<?php

namespace Drupal\syncer\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Database\Connection;
use Symfony\Component\Yaml\Yaml;
use Drupal\Component\Utility\Unicode;
use Drush\Commands\DrushCommands as OriginalDrushCommands;
use Drupal\syncer\BatchExport;
use Drupal\syncer\BatchImport;
use ZipArchive;

/**
 * Drush commands.
 */
class DrushCommands extends OriginalDrushCommands {

  /**
   * A zip directory.
   */
  const ZIP_DIRECTORY = '/syncer/content';

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs DrushCommands object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->directory = dirname(DRUPAL_ROOT . '../') . static::ZIP_DIRECTORY;
  }

  /**
   * Export content.
   *
   * @command syncer:export
   * @validate-module-enabled syncer
   * @aliases sce
   * @usage drush syncer:export ENTITY_TYPE --condition = CONDITION
   */
  public function export($type = '', $options = ['condition' => []]) {
    if (!$type) {
      return $this->io()->error('type is required');
    }

    $operations = [];

    try {
      /** @var mixed $entity_storage */
      $entity_storage = $this->entityTypeManager->getStorage($type);
      /** @var \Drupal\Core\Entity\Query\Sql\Query $query */
      $query = $entity_storage->getQuery();

      if (isset($options['condition']) && is_array($options['condition'])) {
        foreach ($options['condition'] as $condition) {
          $condition = explode(':', $condition);

          if (count($condition) > 1) {
            if ($condition[1][0] === '[' && $condition[1][Unicode::strlen($condition[1]) - 1] === ']') {
              $query->condition($condition[0], explode(',', trim($condition[1], '[]')), $condition[2]);
            }
            else {
              $query->condition($condition[0], $condition[1], $condition[2]);
            }
          }
        }
      }

      $data = $query->execute();

      if ($data && is_array($data) && file_prepare_directory($this->directory, FILE_CREATE_DIRECTORY)) {
        $this->directory = sprintf('%s/%s.zip', $this->directory, $type);
        $zip = fopen($this->directory, 'w');

        foreach ($data as $id) {
          /** @var mixed $entity_storage */
          $operations[] = [
            [BatchExport::class, 'process'],
            [$id, $entity_storage, $this->directory],
          ];
        }

        fclose($zip);
      }
    }
    catch (\Exception $e) {
      $this->io()->error($e);
    }

    batch_set([
      'operations' => $operations,
      'finished' => [BatchExport::class, 'finished'],
    ]);

    drush_backend_batch_process();
  }


  /**
   * Import content.
   *
   * @command syncer:import
   * @validate-module-enabled syncer
   * @aliases sci
   * @usage drush syncer:import ENTITY_TYPE
   */
  public function import($type) {
    /** @var mixed $entity_storage */
    $entity_storage = $this->entityTypeManager->getStorage($type);

    /** @var \ZipArchive $zip */
    $zip = new ZipArchive();
    $zip->open(sprintf('%s/%s.zip', $this->directory, $type));
    $count = $zip->numFiles;

    if (!$count) {
      return $this->io()->warning('nothing to import');
    }

    for ($i = 0; $i < $count; $i++) {
      if ($stat = $zip->statIndex($i)) {
        $content = $zip->getStream($stat['name']);   
        $operations[] = [
          [BatchImport::class, 'process'],
          [Yaml::parse(stream_get_contents($content)), $entity_storage, NULL],
        ];
      }
    }

    $zip->close();

    batch_set([
      'operations' => $operations,
      'finished' => [BatchImport::class, 'finished'],
    ]);

    drush_backend_batch_process();
  }

}
